import { Component, OnInit } from '@angular/core';
import { BPMNDelegateMock } from 'src/app/models/bpmn-delegate-mock';
import { FormBuilder, FormGroup, Validators, FormArray } from '@angular/forms';
import { MockService } from 'src/app/services/mock.service';
import { Router, ActivatedRoute } from '@angular/router';
import ValidationUtils from 'src/app/utils/validation';
import { FlowService } from 'src/app/services/flow.service';
import { BPMNFlowElement } from 'src/app/models/bpmn-flow-element';

@Component({
  selector: 'app-new-manual-mock',
  templateUrl: './new-manual-mock.component.html',
  styleUrls: ['./new-manual-mock.component.css']
})
export class NewManualMockComponent implements OnInit {

  isMockLoading: boolean;
  mock: BPMNDelegateMock;
  actualProject: number;
  actualMock: string;
  delegateServiceTasks: Array<BPMNFlowElement>;

  isEdit: boolean;

  isLoading: boolean;

  public formGroup: FormGroup;

  constructor(private _fb: FormBuilder, private flowService: FlowService, private mockService: MockService, private router: Router, private route: ActivatedRoute) {
    this.route.params.subscribe(params => {
      this.actualProject = params['projectId'];
      this.actualMock = params['mockId'];
    })
  }

  async ngOnInit() {
    this.isLoading = true;

    await this.loadDelegateServiceTasks();

    if (this.actualMock) {
      this.isEdit = true;
      await this.loadMock();
    } else {
      this.isEdit = false;

      this.initNewMock();
    }

    this.initFormGroup();
    this.formGroup.setValue(this.mock);

    this.isLoading = false;
  }

  async loadDelegateServiceTasks() {
    try {
      let result = await this.flowService.getDelegateServiceTasks(this.actualProject);
      this.delegateServiceTasks = result;
    } finally {

    }
  }

  async loadMock() {
    this.isMockLoading = true;
    try {
      let result = await this.mockService.getDelegateMock(this.actualProject, this.actualMock);
      this.mock = result;
    } finally {
      this.isMockLoading = false;
    }
  }

  initNewMock() {
    this.mock = {
      name: "",
      serviceName: "",
      delegateClass: "",
      manualMocks: []
    }
  }

  initFormGroup() {
    this.formGroup = this._fb.group({
      name: ['', Validators.compose([Validators.required])],
      serviceName: ['', Validators.compose([Validators.required])],
      delegateClass: ['', Validators.compose([Validators.required])],
      manualMocks: this._fb.array([])
    });

    this.mock.manualMocks.forEach(mock => this.addManualMockToFormGroup());
  }

  addManualMockToFormGroup() {
    let formGroupManualMock = this._fb.group({
      serviceName: ['', Validators.compose([Validators.required])],
      setter: ['', Validators.compose([Validators.required])]
    });

    this.manualMocksForm.push(formGroupManualMock);
  }

  addManualMockToValue() {
    let manualMock = {
      serviceName: "",
      setter: ""
    }

    this.mock.manualMocks.push(manualMock);
  }

  get manualMocksForm(): FormArray {
    return <FormArray>this.formGroup.get('manualMocks');
  }
  
  addManualMock() {
    this.addManualMockToFormGroup();
    this.addManualMockToValue();
  }

  deleteManualMock(manualMockId: number) {
    this.manualMocksForm.removeAt(manualMockId);
    this.mock.manualMocks.splice(manualMockId, 1);
  }

  async save() {
    if (!this.formGroup.invalid) {

      const formValue = this.formGroup.value;

      console.log(formValue);

      if (this.isEdit) {
        await this.updateTestcase(formValue);
      } else {
        await this.saveTestcase(formValue);
      }
    } else {
      ValidationUtils.validateAllFormFields(this.formGroup);
    }
  }

  async saveTestcase(formValue: BPMNDelegateMock) {
    try {
      await this.mockService.createDelegateMock(this.actualProject, formValue);
      this.router.navigate(['../'], { relativeTo: this.route });
    } catch (err) {
      if (err.status === 409) {
        this.formGroup.controls['name'].setErrors({ 'mockIdTaken': true });
      } else {
        throw err;
      }
    } finally {
      //
    }
  }

  async updateTestcase(formValue: BPMNDelegateMock) {
    try {
      await this.mockService.updateDelegateMock(this.actualProject, this.actualMock, formValue);
      this.router.navigate(['../../'], { relativeTo: this.route });
    } catch (err) {
      if (err.status === 409) {
        this.formGroup.controls['name'].setErrors({ 'mockIdTaken': true });
      } else {
        throw err;
      }
    } finally {
      //
    }
  }
}
