import { Component, OnInit, ViewChild, ViewChildren } from '@angular/core';
import { ActivatedRoute } from '@angular/router';


import { FileService } from 'src/app/services/file.service';
import { ProjectService } from 'src/app/services/project.service';
import { SingleProjectConfig } from 'src/app/models/single-project-config';

//import * as BpmnModdle from 'bpmn-moddle';
let BpmnModdle = require('bpmn-moddle').default;
//import BpmnModdle from 'bpmn-moddle';
import { diff } from 'bpmn-js-differ';
import { BpmnDiagramComponent } from '../bpmn-diagram/bpmn-diagram.component';

@Component({
  selector: 'app-update-project',
  templateUrl: './update-project.component.html',
  styleUrls: ['./update-project.component.css']
})
export class UpdateProjectComponent implements OnInit {

  actualProject: number;
  actualProjectConfig: SingleProjectConfig;

  diagram: string;  // current BPMN-Model
  diagramNew: string; // updated BPMN-Model
  diagramsLoaded: boolean = false;

  @ViewChild('backupDiagram') backupDiagram;
  @ViewChild('newDiagram') newDiagram;

  constructor(private fileService: FileService, private projectService: ProjectService, private route: ActivatedRoute) {
    this.route.params.subscribe(params =>
      this.actualProject = params['projectId']
    )

    
  }

  async ngOnInit() {
    await this.loadProject();
    await this.loadDiagram();
    await this.loadDiagramUpdated();
    
  }

  async compareProjects() {
    var currentDiagram, newDiagram;

    try {
      var result = await new BpmnModdle().fromXML(this.diagram);
      currentDiagram = result.rootElement;
    } catch(err){
        console.log('something went wrong!');
    }

    try {
      var result = await new BpmnModdle().fromXML(this.diagramNew);
      newDiagram = result.rootElement;
    } catch(err){
        console.log('something went wrong!');
    }

    var changes = diff(currentDiagram, newDiagram);

    var container = this.backupDiagram.getBpmnContainer();
    var canvas = container.get('canvas');

    for(let element of Object.keys(changes._removed)) {
      canvas.addMarker(element, 'marker-removed');
    }
    
    var newContainer = this.newDiagram.getBpmnContainer();
    var newCanvas = newContainer.get('canvas');

    for(let element of Object.keys(changes._added)) {
      newCanvas.addMarker(element, 'marker-added');
    }
  }

  async loadProject() {
    try {
      let result = await this.projectService.getProject(this.actualProject);
      this.actualProjectConfig = result;
    } 
    catch(e) {
      console.log(e);
    }
  }

  async loadDiagram() {
    try {
      let result = await this.fileService.getBackupBPMNDiagram(this.actualProject);
      this.diagram = result;
    }
    finally {
      this.diagramsLoaded = false;
    }
  }

  async loadDiagramUpdated() {
    try {
      let result = await this.fileService.getBPMNDiagram(this.actualProject);
      this.diagramNew = result;
    }
    finally {
      this.diagramsLoaded = true;
    }

  }
}