import { FormGroup, FormControl, FormArray, AbstractControl } from "@angular/forms";

export default class ValidationUtils {

    static validateAllFormFields(formGroup: FormGroup) {
        Object.keys(formGroup.controls).forEach(field => {
            const control = formGroup.get(field);

            this.validate(control);
        });
    }

    static validate(control: AbstractControl) {
        if (control instanceof FormControl) {
            control.markAsTouched({ onlySelf: true });
        } else if (control instanceof FormGroup) {
            this.validateAllFormFields(control);
        } else if (control instanceof FormArray) {
            control.controls.forEach(controlForm => {
                this.validate(controlForm);
            });
        }
    }

    static noWhitespaceValidator(control: FormControl) {
        const isWhitespace = control.value.indexOf(' ') >= 0;
        const isValid = !isWhitespace;
        return isValid ? null : { 'whitespace': true };
    }
}